package ai.pai.lensman;

import android.app.Application;
import android.content.Context;
import android.widget.Toast;

import com.nostra13.universalimageloader.cache.disc.impl.UnlimitedDiskCache;
import com.nostra13.universalimageloader.cache.disc.naming.Md5FileNameGenerator;
import com.nostra13.universalimageloader.cache.memory.impl.WeakMemoryCache;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;
import com.nostra13.universalimageloader.core.assist.QueueProcessingType;
import com.nostra13.universalimageloader.utils.StorageUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;

import ai.pai.lensman.db.Preferences;
import ai.pai.lensman.utils.Constants;
import ai.pai.lensman.utils.PatchManager;
import ai.pai.lensman.utils.UrlContainer;

public class App extends Application {

    private static Context   mInstance;

    public static Context getAppContext(){
        return mInstance;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mInstance = this.getApplicationContext();
        if(!checkUsbPermission()){
            Toast.makeText(this,"申请usb权限失败",Toast.LENGTH_SHORT).show();
        }
        Preferences.getInstance().setCameraLibExist(checkCameraLibs());
        initImageLoader();
        new PatchManager().patch(this, UrlContainer.PATCH_CONFIG_URL,Constants.APP_PATCH_DIR+ File.separator+"patch-"+BuildConfig.VERSION_NAME+".dex","ai.pai.lensman.patch.PatchesInfoImpl");
        noMedia();
    }

    private boolean checkUsbPermission(){
        Process process = null;
        DataOutputStream os = null;
        try {
            String cmd = "chmod 777 " + getPackageCodePath();
            String cmd2 = "chmod -R 0777 /dev/bus/usb" ;

            process = Runtime.getRuntime().exec("su"); // 切换到root帐号
            os = new DataOutputStream(process.getOutputStream());
            os.writeBytes(cmd + "\n");
            os.writeBytes(cmd2 + "\n");
            os.writeBytes("exit\n");
            os.flush();
            process.waitFor();
            return true;
        } catch (Exception e) {
            return false;
        } finally {
            try {
                if (os != null) {
                    os.close();
                }
                process.destroy();
            } catch (Exception e) {
            }
        }
    }
    public void copyResToSdcard(String name){
        Field[] raw = R.raw.class.getFields();
        for (Field r : raw) {
            try {
                int id=getResources().getIdentifier(r.getName(), "raw", getPackageName());
                if(r.getName().startsWith("lib")){
                    String path=name+"/"+r.getName()+".so";
                    BufferedOutputStream bufEcrivain = new BufferedOutputStream((new FileOutputStream(new File(path))));
                    BufferedInputStream VideoReader = new BufferedInputStream(getResources().openRawResource(id));
                    byte[] buff = new byte[20*1024];
                    int len;
                    while( (len = VideoReader.read(buff)) > 0 ){
                        bufEcrivain.write(buff,0,len);
                    }
                    bufEcrivain.flush();
                    bufEcrivain.close();
                    VideoReader.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }

    private boolean checkCameraLibs(){
        if(Preferences.getInstance().isCameraLibExist()){
            return true;
        }
        String tmpLibDirPath = "/mnt/sdcard/lensman/so";
        File file = new File(tmpLibDirPath);
        file.mkdirs();
        copyResToSdcard(tmpLibDirPath);
        Process process = null;
        DataOutputStream os = null;
        try {
            String cmd = "chmod 777 " + getPackageCodePath();
            String cmd2 = "chmod -R 0777 /dev/bus/usb" ;

            String cmd3 = "mount -o rw,remount /dev/block/mtdblock8 /system" ;
            String cmd5 = "cp -f  /mnt/sdcard/lensman/*.so  /data/data/ai.pai.lensman" ;
            String cmd6 = "chmod 0777 /data/data/ai.pai.lensman/*.so" ;

            process = Runtime.getRuntime().exec("su"); // 切换到root帐号
            os = new DataOutputStream(process.getOutputStream());
            os.writeBytes(cmd + "\n");
            os.writeBytes(cmd2 + "\n");
            os.writeBytes(cmd3 + "\n");
            os.writeBytes(cmd5 + "\n");
            os.writeBytes(cmd6 + "\n");
            os.writeBytes("exit\n");
            os.flush();
            process.waitFor();
            return true;
        } catch (Exception e) {
             return false;
        } finally {
            try {
                if (os != null) {
                    os.close();
                }
                process.destroy();
            } catch (Exception e) {
            }
        }
    }


    private void noMedia(){
        File noMedia = new File(Constants.APP_ROOT_DIR,".nomedia");
        if(noMedia.exists()){
            return;
        }
        try {
            File rootDir = new File(Constants.APP_ROOT_DIR);
            rootDir.mkdirs();
            noMedia.createNewFile();
        }catch (IOException e){
            e.printStackTrace();
        }
    }

    private void initImageLoader() {

        ImageLoaderConfiguration config = new ImageLoaderConfiguration.Builder(this)
                .memoryCacheExtraOptions(320, 320)
                .diskCacheExtraOptions(320, 320, null)
                .threadPriority(Thread.NORM_PRIORITY - 2)
                .denyCacheImageMultipleSizesInMemory()
                .diskCacheFileNameGenerator(new Md5FileNameGenerator())
                .diskCache(new UnlimitedDiskCache(StorageUtils.getOwnCacheDirectory(this, Constants.TMP_DIR_NAME)))
                .diskCacheSize(50 * 1024 * 1024).tasksProcessingOrder(QueueProcessingType.LIFO)
                .memoryCache(new WeakMemoryCache()).memoryCacheSizePercentage(20)
                .threadPoolSize(4)
                .build();
        ImageLoader.getInstance().init(config);
    }

}
